/* ESI AIS Parser
 * 
 * Copyright 2011/2012 by Pierre van de Laar & Pierre America (Embedded Systems Institute)
 * Copyright 2008 by Brian C. Lane <bcl@brianlane.com>
 * All Rights Reserved
 * 
 */
package nl.esi.metis.aisparser;

/** This interface represents an AIS message of type 18: Standard Class B equipment position report.
*
* @author Pierre van de Laar
* @author Pierre America
* @author Brian C. Lane
*/
public interface AISMessage18 extends AISMessage {

	/** Returns the first set of spare bits.
	 * @return the value of the first set of spare bits. This should be zero.
	 */
	public int getSpare1();

	/** Returns the speed over ground.
	 * @return an integer value in the range of 0 to 1023, 
	 * representing the speed over ground in 1/10 knot steps (0-102.2 knots) <br>
	 * 1023 = not available <br>
	 * 1022 = 102.2 knots or higher
	 */
	public int getSpeedOverGround();

	/** Returns the position accuracy.
	 * @return a boolean value representing position accuracy: <br>
	 * true = high (&le; 10 m) <br>
	 * false = low (>10 m)
	 */
	public boolean getPositionAccuracy();

	/** Returns the longitude. This value can be analyzed further with utility class {@link UtilsLongitude28}.
	 * @return an integer value representing the longitude in 1/10000 minute (&plusmn;180&deg;, East = positive, West = negative). <br>
	 * 181&deg; = (6791AC0 hex) = not available
	 */
	public int getLongitude();

	/** Returns the latitude. This value can be analyzed further with utility class {@link UtilsLatitude27}.
	 * @return an integer value representing the latitude in 1/10000 minute (&plusmn;90&deg;, North = positive, South = negative). <br>
	 *  91&deg; = (3412140 hex) = not available
	 */
	public int getLatitude();

	/** Returns the course over ground. This value can be analyzed further with utility class {@link UtilsCourseOverGround12}.
	 * @return an integer value representing the course over ground in 1/10&deg; for values in the range of 0 to 3599.<br>
	 * 3600 (E10h) = not available. <br>
	 * 3601 or higher should not be used
	 */
	public int getCourseOverGround();

	/** Returns the true heading.
	 * @return an integer value representing the true heading in degrees (0-359). <br>
	 * 511 indicates not available
	 */
	public int getTrueHeading();

	/** Returns the time stamp contained in the message.
	 * This can be analyzed further using utility class {@link UtilsTimeStamp}.
	 * @return an integer value representing the UTC second when the report was generated by the electronic position fixing system (EPFS) (0-59) <br>
	 * 60 if time stamp is not available <br>
	 * 61 if positioning system is in manual input mode <br>
	 * 62 if electronic position fixing system operates in estimated (dead reckoning) mode <br>
	 * 63 if the positioning system is inoperative
	 */
	public int getTimeStamp();

	/** Returns the second set of spare bits.
	 * @return the value of the second set of spare bits. This should be zero.
	 */
	public int getSpare2();

	/** Returns the class B unit flag, which indicates the kind of AIS unit. 
	 * @return a boolean value representing the class B unit flag: <br>
	 * false = Class B SOTDMA unit <br>
	 * true = Class B "CS" unit
	 */
	public boolean getClassBUnitFlag();

	/** Returns the class B display flag, which indicates whether the unit is equipped with an integrated display displaying Message 12 and 14.
	 * @return a boolean value representing the class B display flag: <br>
	 * false = No display available; not capable of displaying Message 12 and 14 <br>
	 * true = Equipped with integrated display displaying Message 12 and 14
	 */
	public boolean getClassBDisplayFlag();

	/** Returns the class B DSC flag, which indicates whether the unit has the digital selective calling (DSC) function.
	 * @return a boolean value representing the class B DSC flag: <br>
	 * false = Not equipped with DSC function <br>
	 * true = Equipped with DSC function (dedicated or time-shared)
	 */
	public boolean getClassBDSCFlag();

	/** Returns the class B band flag, which indicates over which radio bands the unit can communicate.
	 * @return a boolean value representing the class B band flag: <br>
	 * false = Capable of operating over the upper 525 kHz band of the marine band <br>
	 * true = Capable of operating over the whole marine band <br>
	 * (irrelevant if "Class B Message 22 flag" is 0)
	 */
	public boolean getClassBBandFlag();

	/** Returns the class B Message 22 flag, which indicates whether the unit supports frequency management via Message 22.
	 * @return a boolean value representing the class B Message 22 flag: <br>
	 * false = No frequency management via Message 22, operating on AIS1, AIS2 only <br>
	 * true = Frequency management via Message 22
	 */
	public boolean getClassBMessage22Flag();

	/** Returns the assigned mode flag.
	 * @return a boolean value representing the assigned mode flag: <br>
	 * false = Station operating in autonomous and continuous mode <br>
	 * true = Station operating in assigned mode
	 */
	public boolean getAssignedModeFlag();

	/** Returns the RAIM flag, which describes the receiver autonomous integrity monitoring status of the electronic position fixing device.
	 * @return a boolean value: <br>
	 * false = RAIM not in use <br>
	 * true = RAIM in use
	 */
	public boolean getRaimFlag();

	/** Returns the communication state selector flag.
	 * @return a boolean value representing the communication state selector flag: <br>
	 * false = SOTDMA communication state: {@link #getCommunicationState()} will return an {@link Sotdma} object. <br>
	 * true = ITDMA communication state: {@link #getCommunicationState()} will return an {@link Itdma} object. <br>
	 * (always true for Class-B "CS" units)
	 */
	public boolean getCommunicationStateSelectorFlag();

	/** Returns the communication state.
	 * @return the value of the communication state. 
	 * Depending on the {@linkplain #getCommunicationStateSelectorFlag communication state selector flag}, 
	 * this may be {@link Itdma} or {@link Sotdma}.
	 */
	public CommunicationState getCommunicationState();

}