package nl.esi.metis.aisparser;
/** ESI AIS Parser
 * 
 * Copyright 2011/2012 by Pierre van de Laar (Embedded Systems Institute)
 * Copyright 2008 by Brian C. Lane <bcl@brianlane.com>
 * All Rights Reserved
 * 
 * @author Pierre van de Laar
 * @author Brian C. Lane
 */


/** AIS Message 18
 * Standard Class B equipment position report
 * 
 * Field Nr     Field Name                          NrOf Bits   (from, to )
 * ------------------------------------------------------------------------
 *  1	messageID                               	   6	(   1,   6)
 *  2	repeatIndicator                         	   2	(   7,   8)
 *  3	userID                                  	  30	(   9,  38)
 *  4	spare1                                  	   8	(  39,  46)
 *  5	speedOverGround                         	  10	(  47,  56)
 *  6	positionAccuracy                        	   1	(  57,  57)
 *  7	longitude                               	  28	(  58,  85)
 *  8	latitude                                	  27	(  86, 112)
 *  9	courseOverGround                        	  12	( 113, 124)
 * 10	trueHeading                             	   9	( 125, 133)
 * 11	timeStamp                               	   6	( 134, 139)
 * 12	spare2                                  	   2	( 140, 141)
 * 13	classBUnitFlag                          	   1	( 142, 142)
 * 14	classBDisplayFlag                       	   1	( 143, 143)
 * 15	classBDSCFlag                           	   1	( 144, 144)
 * 16	classBBandFlag                          	   1	( 145, 145)
 * 17	classBMessage22Flag                     	   1	( 146, 146)
 * 18	assignedModeFlag                               1	( 147, 147)
 * 19	raimFlag                                	   1	( 148, 148)
 * 20	communicationStateSelectorFlag          	   1	( 149, 149)
 * 21	communicationState                      	  19	( 150, 168)
 *                                                  ---- +
 *                       (maximum) number of bits    168
 */
class AISMessage18Impl extends AISMessageImpl implements AISMessage18 {
	public static final int LENGTH = 168;
	
	public static boolean validLength(int length)
	{
		return (length == LENGTH);
	}
	


	private static final int SPARE1_FROM = 39;
	private static final int SPARE1_TO = 46;

	private int spare1;
	/** spare1
	 * @return int value of spare1 (8 bits [39,46])
	 */
	public int getSpare1() { return spare1; }


	private static final int SPEEDOVERGROUND_FROM = 47;
	private static final int SPEEDOVERGROUND_TO = 56;

	private int speedOverGround;
	/** speedOverGround
	 * @return int value of speedOverGround (10 bits [47,56])
	 */
	public int getSpeedOverGround() { return speedOverGround; }


	private static final int POSITIONACCURACY_BITINDEX = 57;

	private boolean positionAccuracy;
	/** positionAccuracy
	 * @return boolean value of positionAccuracy (bit 57)
	 */
	public boolean getPositionAccuracy() { return positionAccuracy; }


	private static final int LONGITUDE_FROM = 58;
	private static final int LONGITUDE_TO = 85;

	private int longitude;
	/** longitude
	 * @return int value of longitude (28 bits [58,85])
	 */
	public int getLongitude() { return longitude; }


	private static final int LATITUDE_FROM = 86;
	private static final int LATITUDE_TO = 112;

	private int latitude;
	/** latitude
	 * @return int value of latitude (27 bits [86,112])
	 */
	public int getLatitude() { return latitude; }


	private static final int COURSEOVERGROUND_FROM = 113;
	private static final int COURSEOVERGROUND_TO = 124;

	private int courseOverGround;
	/** courseOverGround
	 * @return int value of courseOverGround (12 bits [113,124])
	 */
	public int getCourseOverGround() { return courseOverGround; }


	private static final int TRUEHEADING_FROM = 125;
	private static final int TRUEHEADING_TO = 133;

	private int trueHeading;
	/** trueHeading
	 * @return int value of trueHeading (9 bits [125,133])
	 */
	public int getTrueHeading() { return trueHeading; }


	private static final int TIMESTAMP_FROM = 134;
	private static final int TIMESTAMP_TO = 139;

	private int timeStamp;
	/** timeStamp
	 * @return int value of timeStamp (6 bits [134,139])
	 */
	public int getTimeStamp() { return timeStamp; }


	private static final int SPARE2_FROM = 140;
	private static final int SPARE2_TO = 141;

	private int spare2;
	/** spare2
	 * @return int value of spare2 (2 bits [140,141])
	 */
	public int getSpare2() { return spare2; }


	private static final int CLASSBUNITFLAG_BITINDEX = 142;

	private boolean classBUnitFlag;
	/** classBUnitFlag
	 * @return boolean value of classBUnitFlag (bit 142)
	 */
	public boolean getClassBUnitFlag() { return classBUnitFlag; }


	private static final int CLASSBDISPLAYFLAG_BITINDEX = 143;

	private boolean classBDisplayFlag;
	/** classBDisplayFlag
	 * @return boolean value of classBDisplayFlag (bit 143)
	 */
	public boolean getClassBDisplayFlag() { return classBDisplayFlag; }


	private static final int CLASSBDSCFLAG_BITINDEX = 144;

	private boolean classBDSCFlag;
	/** classBDSCFlag
	 * @return boolean value of classBDSCFlag (bit 144)
	 */
	public boolean getClassBDSCFlag() { return classBDSCFlag; }


	private static final int CLASSBBANDFLAG_BITINDEX = 145;

	private boolean classBBandFlag;
	/** classBBandFlag
	 * @return boolean value of classBBandFlag (bit 145)
	 */
	public boolean getClassBBandFlag() { return classBBandFlag; }


	private static final int CLASSBMESSAGE22FLAG_BITINDEX = 146;

	private boolean classBMessage22Flag;
	/** classBMessage22Flag
	 * @return boolean value of classBMessage22Flag (bit 146)
	 */
	public boolean getClassBMessage22Flag() { return classBMessage22Flag; }


	private static final int ASSIGNEDMODEFLAG_BITINDEX = 147;

	private boolean assignedModeFlag;

	/** Returns the assigned mode flag.
	 * @return a boolean value representing the assigned mode flag: <br>
	 * false = Station operating in autonomous and continuous mode <br>
	 * true = Station operating in assigned mode
	 */
	public boolean getAssignedModeFlag() { return assignedModeFlag; }


	private static final int RAIMFLAG_BITINDEX = 148;

	private boolean raimFlag;
	/** raimFlag
	 * @return boolean value of raimFlag (bit 148)
	 */
	public boolean getRaimFlag() { return raimFlag; }


	private static final int COMMUNICATIONSTATESELECTORFLAG_BITINDEX = 149;

	private boolean communicationStateSelectorFlag;
	/** communicationStateSelectorFlag
	 * @return boolean value of communicationStateSelectorFlag (bit 149)
	 */
	public boolean getCommunicationStateSelectorFlag() { return communicationStateSelectorFlag; }


	private static final int COMMUNICATIONSTATE_FROM = 150;
//	private static final int COMMUNICATIONSTATE_TO = 168;

	private CommunicationState communicationState;
	/** communicationState
	 * Note: For Class-B "CS" the communication state should be ITDMA with the following value 1100000000000000110
	 * @return CommunicationState value of communicationState (19 bits [150,168])
	 */
	public CommunicationState getCommunicationState() { return communicationState; }
	
	/** AISMessage 18 constructor
	 * @param content AIS content
	 * @param prov TODO
	 * @precondition validLength(content.length()) && AISMessageBase.getMessageId(content)==18
	 */
	public AISMessage18Impl(Sixbit content, Provenance prov)
	{
		super(content, prov);
		assert(validLength(content.length()));
		assert(getMessageID() == 18);
	
	    spare1 = content.getIntFromTo(SPARE1_FROM,SPARE1_TO);
	    speedOverGround = content.getIntFromTo(SPEEDOVERGROUND_FROM,SPEEDOVERGROUND_TO);
	    positionAccuracy = content.getBoolean(POSITIONACCURACY_BITINDEX);
	    longitude = UtilsTwosComplement.convertFrom28Bits( content.getIntFromTo(LONGITUDE_FROM,LONGITUDE_TO) );
	    latitude = UtilsTwosComplement.convertFrom27Bits( content.getIntFromTo(LATITUDE_FROM,LATITUDE_TO) );
	    courseOverGround = content.getIntFromTo(COURSEOVERGROUND_FROM,COURSEOVERGROUND_TO);
	    trueHeading = content.getIntFromTo(TRUEHEADING_FROM,TRUEHEADING_TO);
	    timeStamp = content.getIntFromTo(TIMESTAMP_FROM,TIMESTAMP_TO);
	    spare2 = content.getIntFromTo(SPARE2_FROM,SPARE2_TO);
	    classBUnitFlag = content.getBoolean(CLASSBUNITFLAG_BITINDEX);
	    classBDisplayFlag = content.getBoolean(CLASSBDISPLAYFLAG_BITINDEX);
	    classBDSCFlag = content.getBoolean(CLASSBDSCFLAG_BITINDEX);
	    classBBandFlag = content.getBoolean(CLASSBBANDFLAG_BITINDEX);
	    classBMessage22Flag = content.getBoolean(CLASSBMESSAGE22FLAG_BITINDEX);
	    assignedModeFlag = content.getBoolean(ASSIGNEDMODEFLAG_BITINDEX);
	    raimFlag = content.getBoolean(RAIMFLAG_BITINDEX);
	    communicationStateSelectorFlag = content.getBoolean(COMMUNICATIONSTATESELECTORFLAG_BITINDEX);
    	if (communicationStateSelectorFlag)
		{
			communicationState = new Itdma(COMMUNICATIONSTATE_FROM, content);
		}
		else
		{
			communicationState = new Sotdma(COMMUNICATIONSTATE_FROM, content);
		}	
    }
}