package nl.esi.metis.aisparser;

import cern.colt.bitvector.BitVector;

/* ESI AIS Parser
 * 
 * Copyright 2011/2012 by Pierre van de Laar, Pierre America (Embedded Systems Institute)
 * Copyright 2008 by Brian C. Lane <bcl@brianlane.com>
 * All Rights Reserved
 * 
 */

/** Implementation of AIS Message 19:  Extended Class B Equipment Position Report.
 * <pre>
 * Field Nr     Field Name                          NrOf Bits   (from, to )
 * ------------------------------------------------------------------------
 *  1           messageID                                  6    (   1,   6)
 *  2           repeatIndicator                            2    (   7,   8)
 *  3           userID                                    30    (   9,  38)
 *  4           spare1                                     8    (  39,  46)
 *  5           speedOverGround                           10    (  47,  56)
 *  6           positionAccuracy                           1    (  57,  57)
 *  7           longitude                                 28    (  58,  85)
 *  8           latitude                                  27    (  86, 112)
 *  9           courseOverGround                          12    ( 113, 124)
 * 10           trueHeading                                9    ( 125, 133)
 * 11           timeStamp                                  6    ( 134, 139)
 * 12           spare2                                     4    ( 140, 143)
 * 13           name                                     120    ( 144, 263)
 * 14           typeOfShipAndCargoType                     8    ( 264, 271)
 * 15           dimension                                 30    ( 272, 301)
 * 16           typeOfElectronicPositionFixingDevice       4    ( 302, 305)
 * 17           raimFlag                                   1    ( 306, 306)
 * 18           dte                                        1    ( 307, 307)
 * 19           assignedModeFlag                           1    ( 308, 308)
 * 20           spare3                                     4    ( 309, 312)
 *                                                      ---- +
 *                           (maximum) number of bits    312
 * </pre>
 * @author Pierre van de Laar
 * @author Pierre America
 * @author Brian C. Lane
 */
class AISMessage19Impl extends AISMessageImpl implements AISMessage19 {
	public static final int LENGTH = 312;
	
	public static boolean validLength(int length)
	{
		return (length == LENGTH);
	}

	private static final int SPARE1_FROM = 39;
	private static final int SPARE1_TO = 46;

	private int spare1;
	/** spare1
	 * @return int value of spare1 (8 bits [39,46])
	 */
	public int getSpare1() { return spare1; }


	private static final int SPEEDOVERGROUND_FROM = 47;
	private static final int SPEEDOVERGROUND_TO = 56;

	private int speedOverGround;
	/** speedOverGround
	 * @return int value of speedOverGround (10 bits [47,56])
	 */
	public int getSpeedOverGround() { return speedOverGround; }


	private static final int POSITIONACCURACY_BITINDEX = 57;

	private boolean positionAccuracy;
	/** positionAccuracy
	 * @return boolean value of positionAccuracy (bit 57)
	 */
	public boolean getPositionAccuracy() { return positionAccuracy; }


	private static final int LONGITUDE_FROM = 58;
	private static final int LONGITUDE_TO = 85;

	private int longitude;
	/** longitude
	 * @return int value of longitude (28 bits [58,85])
	 */
	public int getLongitude() { return longitude; }


	private static final int LATITUDE_FROM = 86;
	private static final int LATITUDE_TO = 112;

	private int latitude;
	/** latitude
	 * @return int value of latitude (27 bits [86,112])
	 */
	public int getLatitude() { return latitude; }


	private static final int COURSEOVERGROUND_FROM = 113;
	private static final int COURSEOVERGROUND_TO = 124;

	private int courseOverGround;
	/** courseOverGround
	 * @return int value of courseOverGround (12 bits [113,124])
	 */
	public int getCourseOverGround() { return courseOverGround; }


	private static final int TRUEHEADING_FROM = 125;
	private static final int TRUEHEADING_TO = 133;

	private int trueHeading;
	/** trueHeading
	 * @return int value of trueHeading (9 bits [125,133])
	 */
	public int getTrueHeading() { return trueHeading; }


	private static final int TIMESTAMP_FROM = 134;
	private static final int TIMESTAMP_TO = 139;

	private int timeStamp;
	/** timeStamp
	 * @return int value of timeStamp (6 bits [134,139])
	 */
	public int getTimeStamp() { return timeStamp; }


	private static final int SPARE2_FROM = 140;
	private static final int SPARE2_TO = 143;

	private int spare2;
	/** spare2
	 * @return int value of spare2 (4 bits [140,143])
	 */
	public int getSpare2() { return spare2; }


	private static final int NAME_FROM = 144;
	private static final int NAME_TO = 263;

	private String name;
	/** name
	 * @return String value of name (120 bits [144,263])
	 */
	public String getName() { return name; }


	private static final int TYPEOFSHIPANDCARGOTYPE_FROM = 264;
	private static final int TYPEOFSHIPANDCARGOTYPE_TO = 271;

	private int typeOfShipAndCargoType;
	/** typeOfShipAndCargoType
	 * @return int value of typeOfShipAndCargoType (8 bits [264,271])
	 */
	public int getTypeOfShipAndCargoType() { return typeOfShipAndCargoType; }


	private static final int DIMENSION_FROM = 272;
	private static final int DIMENSION_TO = 301;

	private BitVector dimension;
	/** dimension
	 * @return BitVector value of dimension (30 bits [272,301])
	 */
	public BitVector getDimension() { return dimension; }


	private static final int TYPEOFELECTRONICPOSITIONFIXINGDEVICE_FROM = 302;
	private static final int TYPEOFELECTRONICPOSITIONFIXINGDEVICE_TO = 305;

	private int typeOfElectronicPositionFixingDevice;
	/** typeOfElectronicPositionFixingDevice
	 * @return int value of typeOfElectronicPositionFixingDevice (4 bits [302,305])
	 */
	public int getTypeOfElectronicPositionFixingDevice() { return typeOfElectronicPositionFixingDevice; }


	private static final int RAIMFLAG_BITINDEX = 306;

	private boolean raimFlag;
	/** raimFlag
	 * @return boolean value of raimFlag (bit 306)
	 */
	public boolean getRaimFlag() { return raimFlag; }


	private static final int DTE_BITINDEX = 307;

	private boolean dte;
	/** dte
	 * @return boolean value of dte (bit 307)
	 */
	public boolean getDte() { return dte; }


	private static final int ASSIGNEDMODEFLAG_BITINDEX = 308;

	private boolean assignedModeFlag;

	/** Returns the assigned mode flag.
	 * @return a boolean value representing the assigned mode flag: <br>
	 * false = Station operating in autonomous and continuous mode <br>
	 * true = Station operating in assigned mode
	 */
	public boolean getAssignedModeFlag() { return assignedModeFlag; }


	private static final int SPARE3_FROM = 309;
	private static final int SPARE3_TO = 312;

	private int spare3;
	/** spare3
	 * @return int value of spare3 (4 bits [309,312])
	 */
	public int getSpare3() { return spare3; }

	/** AISMessage 19 constructor
	 * @param content AIS content
	 * @param prov the provenance of the message
	 * @precondition validLength(content.length()) && AISMessageBase.getMessageId(content)==19
	 */
	public AISMessage19Impl(Sixbit content, Provenance prov)
	{
		super(content, prov);
		assert(validLength(content.length()));
		assert(getMessageID() == 19);

		spare1 = content.getIntFromTo(SPARE1_FROM,SPARE1_TO);
		speedOverGround = content.getIntFromTo(SPEEDOVERGROUND_FROM,SPEEDOVERGROUND_TO);
		positionAccuracy = content.getBoolean(POSITIONACCURACY_BITINDEX);
		longitude = UtilsTwosComplement.convertFrom28Bits( content.getIntFromTo(LONGITUDE_FROM,LONGITUDE_TO) );
		latitude = UtilsTwosComplement.convertFrom27Bits( content.getIntFromTo(LATITUDE_FROM,LATITUDE_TO) );
		courseOverGround = content.getIntFromTo(COURSEOVERGROUND_FROM,COURSEOVERGROUND_TO);
		trueHeading = content.getIntFromTo(TRUEHEADING_FROM,TRUEHEADING_TO);
		timeStamp = content.getIntFromTo(TIMESTAMP_FROM,TIMESTAMP_TO);
		spare2 = content.getIntFromTo(SPARE2_FROM,SPARE2_TO);
		name = UtilsString.stripAtSigns(content.getStringFromTo(NAME_FROM,NAME_TO));
		typeOfShipAndCargoType = content.getIntFromTo(TYPEOFSHIPANDCARGOTYPE_FROM,TYPEOFSHIPANDCARGOTYPE_TO);
		dimension = content.getBitVectorFromTo(DIMENSION_FROM,DIMENSION_TO);
		typeOfElectronicPositionFixingDevice = content.getIntFromTo(TYPEOFELECTRONICPOSITIONFIXINGDEVICE_FROM,TYPEOFELECTRONICPOSITIONFIXINGDEVICE_TO);
		raimFlag = content.getBoolean(RAIMFLAG_BITINDEX);
		dte = content.getBoolean(DTE_BITINDEX);
		assignedModeFlag = content.getBoolean(ASSIGNEDMODEFLAG_BITINDEX);
		spare3 = content.getIntFromTo(SPARE3_FROM,SPARE3_TO);
	}
}
