package nl.esi.metis.aisparser;
/** ESI AIS Parser
 * 
 * Copyright 2011/2012 by Pierre van de Laar (Embedded Systems Institute)
 * Copyright 2008 by Brian C. Lane <bcl@brianlane.com>
 * All Rights Reserved
 * 
 * @author Pierre van de Laar
 * @author Brian C. Lane
 */

/** This is the base class implementation for all AIS position report messages. It implements the fields that they all have in common. 
 * <pre>
 * Field Nr     Field Name                          NrOf Bits   (from, to )
 * ------------------------------------------------------------------------
 *  1	messageID                               	   6	(   1,   6)
 *  2	repeatIndicator                         	   2	(   7,   8)
 *  3	userID                                  	  30	(   9,  38)
 *  4	navigationalStatus                      	   4	(  39,  42)
 *  5	rateOfTurn                              	   8	(  43,  50)
 *  6	speedOverGround                         	  10	(  51,  60)
 *  7	positionAccuracy                        	   1	(  61,  61)
 *  8	longitude                               	  28	(  62,  89)
 *  9	latitude                                	  27	(  90, 116)
 * 10	courseOverGround                        	  12	( 117, 128)
 * 11	trueHeading                             	   9	( 129, 137)
 * 12	timeStamp                               	   6	( 138, 143)
 * 13	specialManoeuvre                        	   2	( 144, 145)
 * 14	spare                                   	   3	( 146, 148)
 * 15	raimFlag                                	   1	( 149, 149)
 * 16	communicationState                      	  19	( 150, 168)
 *                                                  ---- +
 *                       (maximum) number of bits    168
 * </pre>
 */

abstract class AISMessagePositionReportImpl extends AISMessageImpl implements AISMessagePositionReport {
	public static boolean validLength(int length)
	{
		return (length == AISMessagePositionReportImpl.LENGTH);
	}

	private static final int NAVIGATIONALSTATUS_FROM = 39;
	private static final int NAVIGATIONALSTATUS_TO = 42;

	private int navigationalStatus;
	public int getNavigationalStatus() { return navigationalStatus; }


	private static final int RATEOFTURN_FROM = 43;
	private static final int RATEOFTURN_TO = 50;

	private int rateOfTurn;
	public int getRateOfTurn() { return rateOfTurn; }


	private static final int SPEEDOVERGROUND_FROM = 51;
	private static final int SPEEDOVERGROUND_TO = 60;

	private int speedOverGround;
	public int getSpeedOverGround() { return speedOverGround; }


	private static final int POSITIONACCURACY_BITINDEX = 61;

	private boolean positionAccuracy;
	public boolean getPositionAccuracy() { return positionAccuracy; }


	private static final int LONGITUDE_FROM = 62;
	private static final int LONGITUDE_TO = 89;

	private int longitude;
	public int getLongitude() { return longitude; }


	private static final int LATITUDE_FROM = 90;
	private static final int LATITUDE_TO = 116;

	private int latitude;
	public int getLatitude() { return latitude; }


	private static final int COURSEOVERGROUND_FROM = 117;
	private static final int COURSEOVERGROUND_TO = 128;

	private int courseOverGround;
	public int getCourseOverGround() { return courseOverGround; }


	private static final int TRUEHEADING_FROM = 129;
	private static final int TRUEHEADING_TO = 137;

	private int trueHeading;
	public int getTrueHeading() { return trueHeading; }


	private static final int TIMESTAMP_FROM = 138;
	private static final int TIMESTAMP_TO = 143;

	private int timeStamp;
	public int getTimeStamp() { return timeStamp; }


	private static final int SPECIALMANOEUVRE_FROM = 144;
	private static final int SPECIALMANOEUVRE_TO = 145;

	private int specialManoeuvre;
	public int getSpecialManoeuvre() { return specialManoeuvre; }


	private static final int SPARE_FROM = 146;
	private static final int SPARE_TO = 148;

	private int spare;
	public int getSpare() { return spare; }


	private static final int RAIMFLAG_BITINDEX = 149;

	private boolean raimFlag;
	public static final int LENGTH = 168;
	public boolean getRaimFlag() { return raimFlag; }


	protected static final int COMMUNICATIONSTATE_FROM = 150;

	/** AISMessage Position Report constructor
	 * @param content AIS content
	 * @param prov the provenance of the message
	 * @precondition validLength(content.length())
	 */
	protected AISMessagePositionReportImpl(Sixbit content, Provenance prov)
	{
		super(content, prov);
		assert(validLength(content.length()));
		
	    navigationalStatus = content.getIntFromTo(NAVIGATIONALSTATUS_FROM,NAVIGATIONALSTATUS_TO);
	    rateOfTurn = UtilsTwosComplement.convertFrom8Bits( content.getIntFromTo(RATEOFTURN_FROM,RATEOFTURN_TO) );
	    speedOverGround = content.getIntFromTo(SPEEDOVERGROUND_FROM,SPEEDOVERGROUND_TO);
	    positionAccuracy = content.getBoolean(POSITIONACCURACY_BITINDEX);
	    longitude = UtilsTwosComplement.convertFrom28Bits( content.getIntFromTo(LONGITUDE_FROM,LONGITUDE_TO) );
	    latitude = UtilsTwosComplement.convertFrom27Bits( content.getIntFromTo(LATITUDE_FROM,LATITUDE_TO) );
	    courseOverGround = content.getIntFromTo(COURSEOVERGROUND_FROM,COURSEOVERGROUND_TO);
	    trueHeading = content.getIntFromTo(TRUEHEADING_FROM,TRUEHEADING_TO);
	    timeStamp = content.getIntFromTo(TIMESTAMP_FROM,TIMESTAMP_TO);
	    specialManoeuvre = content.getIntFromTo(SPECIALMANOEUVRE_FROM,SPECIALMANOEUVRE_TO);
	    spare = content.getIntFromTo(SPARE_FROM,SPARE_TO);
	    raimFlag = content.getBoolean(RAIMFLAG_BITINDEX);
	}
}