/* ESI AIS Parser
 * 
 * Copyright 2011/2012 by Pierre van de Laar & Pierre America (Embedded Systems Institute)
 * Copyright 2008 by Brian C. Lane <bcl@brianlane.com>
 * All Rights Reserved
 * 
 */
package nl.esi.metis.aisparser;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Arrays;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/** This class supports parsing of AIS messages.
 * It can be used as follows:
 * <ul>
 * <li> Define a class that implements interface {@link IHandleAISMessage}. This should do something useful with the AIS messages that it gets.</li>
 * <li> Create an instance of {@link AISParser}, using an instance of the above AIS message handler class.</li>
 * <li> Feed the parser with lines, either using the {@link #readFile readFile} method or one line at a time using the {@link #handleLine handleLine} method. 
 *      Now each time an AIS message is complete, it will be fed to the AIS message handler.</li>
 * </ul>
 * See the {@link #main main} method for a simple example.
 * @author Pierre America
 * @author Pierre van de Laar
 */
public class AISParser {
	/** The line handler that will be invoked for each line that is fed to the parser. */
	private IHandleLine lineHandler = null;
	
	/** Constructs an AIS parser object.
	 * This object will use the provided AIS message handler to deal with every AIS message it will find.
	 * @param aisMessageHandler a handler for AIS messages.
	 */
	public AISParser (IHandleAISMessage aisMessageHandler) {
		HandlerAIVDMMessage aivdmMessageHandler = new HandlerAIVDMMessage(aisMessageHandler);
		HandlerAIVDMLine aivdmLineHandler = new HandlerAIVDMLine(aivdmMessageHandler);
		lineHandler = new HandlerLine(aivdmLineHandler);
	}
	
	/** A pattern that matches a line in a file with a time stamp and a raw AIS message, used by the {@link #readFile readFile} method.
	 * The predefined pattern will fit most reasonable file formats. 
	 * If it does not work for you, you can always change it. */
	public Pattern linePattern = Pattern.compile(".*?(\\d+(?:\\.\\d*)?)?\\D*(!(?:AI|BS)VD[MO].*)");
	
	/** Reads a file with AIS messages and handles all individual messages in the file.
	 * @param file the file to be processed
	 */
	public void readFile(File file) {
		try {
			FileReader input = new FileReader(file);
			BufferedReader bufRead = new BufferedReader(input);
			double lastTime = 0;
			int lineCount = 0;

			String line; // String that holds current file line
			do {
				line = bufRead.readLine();
				if (line != null) {
					lineCount++;
					Matcher m = linePattern.matcher(line);
					if (m.matches())
					{
						assert (m.groupCount() == 2);
						if (m.group(1) != null){
							lastTime = Double.parseDouble(m.group(1));
						}
						FileSource fs = new FileSource(file, lineCount, lastTime);
						lineHandler.handleLine(fs, m.group(2));
					}
					else
					{
						System.err.println("Illegal line " + line);
					}
				}
			} while (line != null);
			lineHandler.finished();
			bufRead.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/** Handles a single input line.
	 * The line must be passed together with the provenance information.
	 * This is useful for situations where live AIS information arrives spread out over time.
	 * @param p the provenance of the input line
	 * @param line a string containing a raw AIS message
	 */
	public void handleLine(Provenance p, String line) {
		lineHandler.handleLine(p, line);
	}
	
	/** Signals that the handling of individual lines is finished.
	 * This method should be called after a number of lines are passed to the {@link #handleLine(Provenance, String) handleLine} method.
	 */
	public void finished() {
		lineHandler.finished();
	}
	
	/** This is a simple example of how the {@link AISParser} class can be used.
	 * This method just counts the AIS messages of type 1 in a single file or in a whole directory of AIS files.
	 * For this purpose it uses a simple private message handler class.
	 * @param args a file to be parsed, or a directory of files to be parsed
	 */
	public static void main(String[] args) {
		MyAISMessageHandler messageHandler = new MyAISMessageHandler();
		AISParser parser = new AISParser (messageHandler);
		
		String aisPath = args[0];
		File aisFile = new File(aisPath);
		if (aisFile.isFile()) {
			parser.readFile(new File(aisPath));
		} else 
			if (aisFile.isDirectory()){
				File files[] = aisFile.listFiles();
				Arrays.sort(files);

				for (int i = 0; i < files.length; i++) {
					if (files[i].isFile()) {
						parser.readFile(files[i]);
					}
				}
			}

		System.out.println("Number of type 1 messages = " + messageHandler.message1Count);
	}
	
	/** This private class illustrates how an AIS message handler can be defined.
	 * @author Pierre America
	 */
	static private class MyAISMessageHandler implements IHandleAISMessage {
		/** The number of AIS messages of type 1 that were found. */
		public int message1Count = 0;

		/** Handles a single AIS message. 
		 * In this case, if the message has type 1 the count is incremented.
		 * @param message the message to be handled
		 */
		public void handleAISMessage(AISMessage message) {
			if (message instanceof AISMessage01) {
				@SuppressWarnings("unused")
				AISMessage01 msg1 = (AISMessage01) message; // purely for illustrating how to handle individual message types
				message1Count++;
			}
		}
	}

}
