/* ESI AIS Parser
 * 
 * Copyright 2011/2012 by Pierre van de Laar & Pierre America (Embedded Systems Institute)
 * Copyright 2008 by Brian C. Lane <bcl@brianlane.com>
 * All Rights Reserved
 * 
 */
package nl.esi.metis.aisparser;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/** AIVDMLine
 *
 */
public class AIVDMLine 
{
	private final static String templateNrOfParts = "[1-9]"; 		//There can be only 9 parts
	private final static String templatePartId = "[1-9]"; 			//PartId <= NrOfParts
	private final static String templateMessageId = "\\d?";			//Message Id [0-9]
	
	/*
	 * According to the Standard ITU-R M.1371-4 channel is a single character with value either A or B
	 * Yet, paragraph 4.4.2 on page 93 mentions four frequencies.
	 * And, paragraph 4.2.1 on page 77 mentions regional frequencies and default channel 1 and 2
	 * 
	 *  In practice, also '1','2' and 'C' is observed.
	 *  Furthermore, also empty string is observed.
	 */
	private final static String templateTalkerID = "(?:AI|BS)";		// known talker ids  	 	
	private final static String templateChannel = "[ABC12]?";  	 	
	private final static String templatePayLoad = "[0-W`-w]+"; 		//TODO: is there a maximal length?
	private final static String templateNrOfFillBits = "[0-5]";		//FillBits: one character has 6 bits -> maximal 5 fill bits 
	private final static String templateCheckSum = "[0-9A-F]{2}";	//Two hexadecimal values: never observed lowercase

	private final static Pattern VdmPattern = Pattern.compile("!(" 
	        + templateTalkerID
			+ "VD[MO],("											//external and own ship messages 
            + templateNrOfParts +"),("
            + templatePartId +"),("
            + templateMessageId +"),("
            + templateChannel + "),("
            + templatePayLoad +"),("
            + templateNrOfFillBits + "))\\*(" 
            + templateCheckSum +")");


	private final static int indexPartCheckSum = 1;
	private final static int indexNrOfParts = 2;
	private final static int indexPartId = 3;
	private final static int indexMessageId = 4;
	private final static int indexChannel = 5;
	private final static int indexPayLoad = 6;
	private final static int indexNrOfFillBits = 7;
	private final static int indexCheckSum = 8;
	
	private Matcher matcher;
	public AIVDMLine(Provenance provenance, String s)
	{
		this.provenance = provenance;
		matcher = VdmPattern.matcher(s); 
	}
	
	private Provenance provenance;
	/** provenance so far
	 * @return the provenance of the AIVDM message so far
	 */
	public Provenance getProvenance() {
		return provenance;
	}

	
	/**Is AIVDM Line syntactically valid?
	 * @return boolean representing the syntactical validity
	 */
	public boolean isSyntacticallyValid()
    {
    	return matcher.matches();
    }
	
	/** Get Number Of Parts in AIDVM Line
	 * @precondition isSyntacticallyValid()
	 * @return number of parts 
	 */
	public int nrOfParts()
	{
		assert(isSyntacticallyValid());
		return Integer.parseInt(matcher.group(indexNrOfParts));
	}

	/** Get Part Id in AIDVM Line
	 * @precondition isSyntacticallyValid()
	 * @return part identifier 
	 */
	public int partId() {
		assert(isSyntacticallyValid());
		return Integer.parseInt(matcher.group(indexPartId));
	}
	
	/** Get Message Id in AIDVM Line
	 * @precondition isSyntacticallyValid()
	 * @postcondition messageId().length() <= 1
	 * @return message identifier 
	 */
	public String messageId() {
		assert(isSyntacticallyValid());
		String retval = matcher.group(indexMessageId);
		assert(retval.length() <= 1);
		return retval;
	}

	/** Get Channel in AIDVM Line
	 * according to standard the channel is a single character with value A or B
	 * But in practice also 1 , 2 and even C is observed
	 * Furthermore, the channel is sometimes missing resulting in an empty string 
	 * @precondition isSyntacticallyValid()
	 * @postcondition channelId().length() <= 1
	 * @return channel 
	 */
	public String channelId() {
		assert(isSyntacticallyValid());
		String retval = matcher.group(indexChannel);
		assert(retval.length() <= 1);
		return retval;
	}
	
	/** Get PayLoad in AIDVM Line
	 * @precondition isSyntacticallyValid()
	 * @postcondition 
	 * @return pay load 
	 */
	public String payLoad() {
		assert(isSyntacticallyValid());
		return matcher.group(indexPayLoad);
	}
	
	/** Get NrOf FillBits in AIDVM Line
	 * According to the standard the NrOf FillBits is between 0 and 5
	 * @precondition isSyntacticallyValid()
	 * @return number of FillBits 
	 */
	public int nrOfFillBits() {
		assert(isSyntacticallyValid());
		return Integer.parseInt(matcher.group(indexNrOfFillBits));
	}

	/** Get CheckSum in AIDVM Line
	 * @precondition isSyntacticallyValid()
	 * @postcondition 0 <= return value < 256
	 * @return message identifier 
	 */
	public int checkSum() {
		assert(isSyntacticallyValid());
		int retval = Integer.parseInt(matcher.group(indexCheckSum),16);
		assert(0 <= retval);
		assert(retval < 256);
		return retval;
	}

	/**Is AIVDM Line semantically valid?
	 * @precondition isSyntacticallyValid()
	 * @return boolean representing the semantical validity
	 */
	public boolean isSemanticallyValid()
    {
		assert(isSyntacticallyValid());
    	return isCheckSumCorrect() && isPartIdCorrect();
    }

	/**Is CheckSum correct?
	 * @precondition isSyntacticallyValid()
	 * @return boolean representing the equality of the parsed and calculated checksum
	 */
	public boolean isCheckSumCorrect()
    {
    	return checkSum() == calculateCheckSum();
    }
	
	/**Calculate Checksum
	 * @precondition isSyntacticallyValid()
	 * @return calculated checksum
	 */
	private int calculateCheckSum()
    {
		int checksum = 0;
		String part = matcher.group(indexPartCheckSum); 
		for (int i = 0; i < part.length(); i++)
		{
			checksum ^= part.charAt(i);
		}
		return checksum; 
    }

	/**Is PartId correct?
	 * @precondition isSyntacticallyValid()
	 * @return boolean representing whether partId() <= nrOfParts() 
	 */
	public boolean isPartIdCorrect()
    {
    	return partId() <= nrOfParts();
    }

	/**Is Valid?
	 * @return boolean representing whether Line is valid AIVDMLine 
	 */
	public boolean isValid()
    {
    	return isSyntacticallyValid() && isSemanticallyValid();
    }
}