/* ESI AIS Parser
 * 
 * Copyright 2011/2012 by Pierre van de Laar & Pierre America (Embedded Systems Institute)
 * Copyright 2008 by Brian C. Lane <bcl@brianlane.com>
 * All Rights Reserved
 * 
 */
package nl.esi.metis.aisparser;

public class AIVDMMessage {
	private boolean isComplete;
	private int nrOfParts;
	private int lastPartId;
	private String msgId;
	private String channelId;
	private String[] messages;
	private Provenance[] provenances;
	private int nrOfFillBits;
	
	/** Get Number Of Parts of AIDVM Message
	 * @return number of parts 
	 */
	public int nrOfParts()
	{
		return nrOfParts;
	}

	/** Get Last Part Id of AIDVM Message
	 * @return part identifier 
	 */
	public int lastPartId() {
		return lastPartId;
	}
	
	/** Get Message Id of AIDVM Message
	 * @return message identifier 
	 */
	public String messageId() {
		return msgId;
	}

	/** Get Channel Identifier of AIDVM Message
	 * @return channel id 
	 */
	public String channelId() {
		return channelId;
	}

	/** Composite message so far
	 * @return the content of the AIVDM message so far
	 * @precondition isComplete()
	 */
	public String getMessage() {
		assert(isComplete);
		StringBuffer sb = new StringBuffer();
		for(int i=0; i < messages.length; i++)
		{
			sb.append(messages[i]);
		}
		return  sb.toString();
	}
	
	/** provenance so far
	 * @return the provenance of the AIVDM message so far
	 */
	public Provenance getProvenance() {
		return  new AIVDMMessageCombination(provenances);
	}
	
	/** Get number of fill bits of AIDVM Message
	 * @return number of fill bits
	 */
	//TODO: is it correct to only look at the last fill bits specified?
	public int nrOfFillBits() {
		return nrOfFillBits;
	}

    /** Constructor
	 * @precondition line.isValid() 
	 * @param line
	 */
	public AIVDMMessage(AIVDMLine line)
	{
		assert(line.isValid());
		
		nrOfParts = line.nrOfParts();
		lastPartId = line.partId();
		msgId = line.messageId();
		channelId = line.channelId();
		messages = new String[nrOfParts];
		messages[lastPartId-1] = line.payLoad();	//TODO: deep copy needed?
		nrOfFillBits = line.nrOfFillBits();
	
		provenances = new Provenance[nrOfParts];
		provenances[lastPartId-1]= line.getProvenance();
	
		isComplete = (lastPartId == 1);
	}

	/**Is AIVDM message complete?
	 * @return boolean indicating whether message is complete (so far)
	 */
	public boolean isComplete()
	{
		return isComplete;
	}
	
	/** Is the last part of the AIVDM message added? 
	 * @return true if the last part has been added, false otherwise
	 */
	public boolean atEnd()
	{
		return lastPartId == nrOfParts;
	}
	
	/** Is line part of this AIVDM message
	 * Same number of parts, same channelId, and same messageId, incremental/sequential partId 
	 * @param line
	 * @return boolean indicating whether line is next part of AIVDM message
	 */
	public boolean isPartOfMessage(AIVDMLine line)
	{
		return ( (nrOfParts == line.nrOfParts()) &&
		         (lastPartId < line.partId()) &&
		         channelId.equals(line.channelId()) &&
		         msgId.equals(line.messageId()) );
	}

	/** Add part to AIVDM Message
	 * @param line
	 */
	public void add(AIVDMLine line)
	{
		final int partId = line.partId();
		isComplete &= (lastPartId+1 == partId);
		lastPartId = partId;
		messages[lastPartId-1] = line.payLoad();
		provenances[lastPartId-1]=line.getProvenance();
		nrOfFillBits = line.nrOfFillBits();		//only last nrOfFillBits is relevant  (all earlier should be zero)
	}
}