package nl.esi.metis.aisparser;

import cern.colt.bitvector.BitVector;

/**
 * 6-bit packed ASCII functions
 * Copyright 2006-2008
 * @author Brian C. Lane <bcl@brianlane.com
 * @author Pierre van de Laar
 * All Rights Reserved
 *
 */


/**
 * This class's methods are used to extract data from the 6-bit packed
 * ASCII string used by AIVDM/AIVDO AIS messages.
 */

public class Sixbit {
	/**
	 * Number of BITS obtained from a single Character
	 */
	public static final int BITSPERCHAR = 6;
	
	/**
	 * Number of BITS needed to get a single Character
	 */
	public static final int BITSNEEDEDFORCHAR = 6;
	
	private String bits;          //!< raw 6-bit ASCII data string
	private BitVector bv;
    private int nrOfFillBits;	      //!< Number of padding bits at end
    
    public static boolean isValidChar(char ascii)
    {
    	return ((ascii >= 0x30) && (ascii <= 0x77)) && ((ascii <= 0x57) || (ascii >= 0x60));
    }
    
    public static boolean isValidString(String bits)
    {
    	//bounded linear search
    	int x = 0;
    	int y = bits.length();
    	while (x != y)
    	{
    		if (isValidChar(bits.charAt(x)))
    		{
    			x++;
    		}
    		else
    		{
    			y = x;
    		}
    	}
    	return x == bits.length();
    }
    
    /** Constructor
     * @param bits
     * @param nrOfFillBits
     * @precondition isValidString (bits)
     * @precondition 0 <= fillBits <= 5
     */
    public Sixbit(String bits, int nrOfFillBits) 
    {
    	assert(isValidString(bits));
    	this.bits = bits;
    	bv = new BitVector(bits.length()*BITSPERCHAR);
    	for(int i = 0; i < bits.length(); i++)
    	{
    		bv.putLongFromTo(Sixbit.binfrom6bit(bits.charAt(bits.length()-1-i)), i*BITSPERCHAR, (i+1)*BITSPERCHAR -1);
    	}
    	this.nrOfFillBits = nrOfFillBits;    	
    }
  
    /*
     * Return the number of bits
     * 
     * Takes into account the number of padding bits.
     */
	public int length()
	{
		return ( (bits.length() * BITSPERCHAR) - nrOfFillBits);
	}

	/**
	 * Convert an ASCII value to a 6-bit binary value
	 *  
     * This function checks the ASCII value to make sure it can be converted.
     * If not, it throws an IllegalArgumentException
     * Otherwise it returns the 6-bit binary value.
     * 
     * @param ascii character to convert
     * @precondition isValidChar(char ascii)
     *
     * This is used to convert the packed 6-bit value to a binary value. It
     * is not used to convert data from fields such as the name and 
     * destination -- Use ais2ascii() instead.
     */
	public static int binfrom6bit(char ascii)
	{
		assert (isValidChar(ascii));
		int retval;
		if( ascii < 0x60 )
		{
			retval = (ascii - 0x30);
		}
		else
		{
			retval = (ascii - 0x38);
		}
		assert((retval & 0x3F) == retval);
		return retval;
	}
	
	
	/** Convert a binary value to a 6-bit ASCII value
	 *
  	 * This function checks the binary value to make sure it can be converted.
     * If not, it throws an IllegalArgumentException.
     * Otherwise it returns the 6-bit ASCII value.
     * 
     * @param value to convert
     * @precondition 0 <= value <= 0x3F
     * @return converted value in 6-bit ASCII representation
     * 
	 */
	public static int binto6bit( int value )
	{
		assert (0 <= value);
		assert(value <= 0x3F);

		if (value < 0x28)
			return value + 0x30;
		else
			return value + 0x38;
	}

	
	/** Convert a AIS 6-bit character to ASCII
	 * 
     * @param value 6-bit value to be converted
     *
     * @return corresponding ASCII value (0x20-0x5F)
     *  
     * This function is used to convert binary data to ASCII. This is 
     * different from the 6-bit ASCII to binary conversion for VDM 
     * messages; it is used for strings within the datastream itself.
     * eg. Ship Name, Callsign and Destination.
     */
	public static char ais2ascii( int value )
	{
		assert (0 <= value);
		assert(value <= 0x3F);

		if( value < 0x20 )
			return (char)(value + 0x40);
		else
			return (char)value;
	}

	/**  get boolean From AIS message
	 * @param bitIndex index of bit
	 *
     * This method returns the requested bit interpreted 
     * as a boolean from the AIS message.
	 */
	public boolean getBoolean( int bitIndex)
	{
		return bv.get(bv.size()-bitIndex);
	}
	
	/** get int From AIS message
	 * @param fromIndex begin index (inclusive)
	 * @param toIndex end index (inclusive)
	 * @precondition toIndex - fromIndex + 1 <= 32
	 *
     * This method returns the requested bits interpreted 
     * as an integer (MSB first) from the AIS message.
	 */
	public int getIntFromTo( int fromIndex, int toIndex )
	{
		assert(toIndex-fromIndex < 32);
		
        return (int) bv.getLongFromTo(bv.size()-toIndex, bv.size()-fromIndex);
	}

	/** get BitVector From AIS message
	 * @param fromIndex begin index (inclusive)
	 * @param toIndex end index (inclusive)
	 *
     * This method returns the requested bits  
     * as a BitVector from the AIS message.
	 */
	public BitVector getBitVectorFromTo( int fromIndex, int toIndex )
	{
        return bv.partFromTo(bv.size()-toIndex, bv.size()-fromIndex);
	}

	/** get String From AIS message
	 * @param fromIndex begin index (inclusive)
	 * @param toIndex end index (inclusive)
	 * @precondition (toIndex - fromIndex + 1)%6 == 0
	 *
     * This method returns the requested bits interpreted 
     * as a String from the AIS message.
	 */
	public String getStringFromTo( int fromIndex, int toIndex )
	{
		assert( (toIndex-fromIndex+1)%BITSNEEDEDFORCHAR == 0);
		
		StringBuilder sb = new StringBuilder();
	    	
        /* Get the 6-bit string, convert to ASCII */
    	for (int i=fromIndex; i < toIndex; i+= BITSNEEDEDFORCHAR)
        {
   			int value = getIntFromTo(i, i+BITSNEEDEDFORCHAR-1);
   			sb.append(Sixbit.ais2ascii(value));
	    }
	    return sb.toString();
	}
}