package nl.esi.metis.aisparser;

/** This class provides functions to analyze the (12-bit signed integer) course-over-ground value that is returned by 
 * {@link AISMessagePositionReport#getCourseOverGround()}, {@link AISMessage18#getCourseOverGround()} and {@link AISMessage19#getCourseOverGround()}.
 * @author Pierre van de Laar
 * @author Pierre America
 * @author Brian C. Lane
 */
public class UtilsCourseOverGround12 {
	/** The course-over-ground value that is used to signal unavailability, according to the AIS standard. */
	private static final int DEFAULTVALUE = 0xE10;

	/** The minimum valid course-over-ground value. */
	private static final int MINVALUE = 0 ;

	/** The maximum valid course-over-ground value. */
	private static final int MAXVALUE =  3599;
	
	/** The multiplication factor to convert course-over-ground values to degrees. */
	private static final double UNITCONVERSIONFACTOR = 1.0/10;
  
	/** Checks if the latitude value is semantically correct (according to the standard). 
	 * This means that it is either a valid latitude value or the standard value indicating unavailability.
	 * @param value the course-over-ground value that is returned by {@link AISMessagePositionReport#getCourseOverGround()}.
	 * @return true if the value is semantically correct
	 */
	public static boolean isSemanticallyCorrectValue( int value )
	{
		return ( ( UtilsCourseOverGround12.MINVALUE <= value ) && ( value <= UtilsCourseOverGround12.MAXVALUE ) ) || (value == UtilsCourseOverGround12.DEFAULTVALUE);
	}
	
	/** Checks if the course-over-ground value is available.
	 * This means that it is a value in between the minimum and maximum valid course-over-ground values.
	 * @param value the course-over-ground value that is returned by {@link AISMessagePositionReport#getCourseOverGround()}.
	 * @return true if the course-over-ground is available
	 */
	public static boolean isAvailable( int value )
	{
		return !(value == UtilsCourseOverGround12.DEFAULTVALUE);
	}
	
	/** Converts the course-over-ground value (in 1/10 degrees) to degrees.
	 * @param value the course-over-ground value that is returned by {@link AISMessagePositionReport#getCourseOverGround()}.
	 * @return the course-over-ground  in degrees
	 */
	public static double toDegrees ( int value )
	{
		return value * UtilsCourseOverGround12.UNITCONVERSIONFACTOR; 
	}
}