package nl.esi.metis.aisparser;

/** This class provides functions to analyze the (17-bit signed integer) latitude value that is returned by 
 * {@link AISMessage17#getLatitude}, {@link AISMessage27#getLatitude}, 
 * {@link DestinationArea#latitudeUpperRightCorner}, or {@link DestinationArea#latitudeLowerLeftCorner}.
 * @author Pierre van de Laar
 * @author Pierre America
 * @author Brian C. Lane
 */
public class UtilsLatitude17 {
	/** The number of integer units in a minute (as a measure of angle). 
	 * The fact that this value is 10 means that the integer latitude values we are dealing with express an angle in units of 1/10 minute.	 */
	private static final long UnitsPerMinute = 10;
	
	/** The number of minutes in a degree (as a measure of angle). */
	private static final long MinutesPerDegree = 60;
	
	/** The latitude value that is used to signal unavailability, according to the AIS standard. */
	private static final long DEFAULTVALUE = 91 * MinutesPerDegree * UnitsPerMinute ;	//91 degrees 

	/** The minimum valid latitude value. */
	private static final long MINVALUE = -90 * MinutesPerDegree * UnitsPerMinute; //-90 degrees

	/** The maximum valid latitude value. */
	private static final long MAXVALUE =  90 * MinutesPerDegree * UnitsPerMinute; //+90 degrees

	/** The multiplication factor to convert latitude values to degrees. */
	private static final double MultiplicationFactorToDegrees = 1.0 / (MinutesPerDegree * UnitsPerMinute);

	/** The multiplication factor to convert latitude values to radians. */
	private static final double MultiplicationFactorToRadians = Math.PI / (180 * MinutesPerDegree * UnitsPerMinute);
	
	/** Checks if the latitude value is semantically correct (according to the standard). 
	 * This means that it is either a valid latitude value or the standard value indicating unavailability.
	 * @param value the latitude value that is returned by 
     * {@link AISMessage17#getLatitude}, {@link AISMessage27#getLatitude}, 
     * {@link DestinationArea#latitudeUpperRightCorner}, or {@link DestinationArea#latitudeLowerLeftCorner}.
	 * @return true if the value is semantically correct
	 */
	//TODO: message 17 & 22 mention explicitly a default /not available value, but messages 23 and 27 do not. Should they NOT accept the default value as semantically correct?
	public static boolean isSemanticallyCorrectValue( long value )
	{
		return ( ( UtilsLatitude17.MINVALUE <= value ) && ( value <= UtilsLatitude17.MAXVALUE ) ) || (value == UtilsLatitude17.DEFAULTVALUE);
	}

	/** Checks if the latitude value is available.
	 * This means that it is a value in between the minimum and maximum valid latitude values.
	 * @param value the latitude value that is returned by 
     * {@link AISMessage17#getLatitude}, {@link AISMessage27#getLatitude}, 
     * {@link DestinationArea#latitudeUpperRightCorner}, or {@link DestinationArea#latitudeLowerLeftCorner}.
	 * @return true if the latitude is available
	 */
	public static boolean isAvailable(long value)
	{
		return !(value == UtilsLatitude17.DEFAULTVALUE); 
	}

	/** Converts the latitude value (in 1/10 minutes) to degrees.
	 * @param value the latitude value that is returned by 
     * {@link AISMessage17#getLatitude}, {@link AISMessage27#getLatitude}, 
     * {@link DestinationArea#latitudeUpperRightCorner}, or {@link DestinationArea#latitudeLowerLeftCorner}.
	 * @return the latitude value in degrees
	 */
	public static double toDegrees ( long value )
	{
		return value * UtilsLatitude17.MultiplicationFactorToDegrees; 
	}
	
	/** Converts the latitude value (in 1/10 minutes) to radians.
	 * @param value the latitude value that is returned by 
     * {@link AISMessage17#getLatitude}, {@link AISMessage27#getLatitude}, 
     * {@link DestinationArea#latitudeUpperRightCorner}, or {@link DestinationArea#latitudeLowerLeftCorner}.
	 * @return the latitude value in radians
	 */
	//TODO: should be removed?  use Math.toRadians(angdeg) instead (but with performance overhead?)
	public static double toRadians ( long value )
	{
		return value * UtilsLatitude17.MultiplicationFactorToRadians;
	}
	
	/** This constructor is made private to indicate that clients should not construct instances of this class. */
	private UtilsLatitude17 () {
	}
}