package nl.esi.metis.aisparser;

/** This class provides functions to analyze the (27-bit signed integer) latitude value that is returned by 
 * {@link AISMessagePositionReport#getLatitude}, {@link AISMessageUTCReport#getLatitude}, {@link AISMessage09Impl#getLatitude}, 
 * {@link AISMessage18#getLatitude}, {@link AISMessage19#getLatitude}, or {@link AISMessage21#getLatitude}.
 * @author Pierre van de Laar
 * @author Pierre America
 * @author Brian C. Lane
 */
public class UtilsLatitude27 {
	/** The number of integer units in a minute (as a measure of angle). 
	 * The fact that this value is 10000 means that the integer latitude values we are dealing with express an angle in units of 1/10000 minute.	 */
	private static final int UnitsPerMinute = 10000; //10 000 units/minute

	/** The number of minutes in a degree (as a measure of angle). */
	private static final int MinutesPerDegree = 60; //60 minutes/degree
	
	/** The latitude value that is used to signal unavailability, according to the AIS standard. */
	private static final int DEFAULTVALUE = 91 * MinutesPerDegree * UnitsPerMinute ;	//91 degrees 

	/** The minimum valid latitude value. */
	private static final int MINVALUE = -90 * MinutesPerDegree * UnitsPerMinute; //-90 degrees

	/** The maximum valid latitude value. */
	private static final int MAXVALUE =  90 * MinutesPerDegree * UnitsPerMinute; //+90 degrees

	/** The multiplication factor to convert latitude values to degrees. */
	private static final double MultiplicationFactorToDegrees = 1.0 / (MinutesPerDegree * UnitsPerMinute);

	/** The multiplication factor to convert latitude values to radians. */
	private static final double MultiplicationFactorToRadians = Math.PI / (180 * MinutesPerDegree * UnitsPerMinute);
	
	/** Checks if the latitude value is semantically correct (according to the standard). 
	 * This means that it is either a valid latitude value or the standard value indicating unavailability.
	 * @param value the latitude value that is returned by 
     * {@link AISMessagePositionReport#getLatitude}, {@link AISMessageUTCReport#getLatitude}, {@link AISMessage09Impl#getLatitude}, 
     * {@link AISMessage18#getLatitude}, {@link AISMessage19#getLatitude}, or {@link AISMessage21#getLatitude}.
	 * @return true if the value is semantically correct
	 */
	public static boolean isSemanticallyCorrectValue( int value )
	{
		return ( ( UtilsLatitude27.MINVALUE <= value ) && ( value <= UtilsLatitude27.MAXVALUE ) ) || (value == UtilsLatitude27.DEFAULTVALUE);
	}

	/** Checks if the latitude value is available.
	 * This means that it is a value in between the minimum and maximum valid latitude values.
	 * @param value the latitude value that is returned by 
     * {@link AISMessagePositionReport#getLatitude}, {@link AISMessageUTCReport#getLatitude}, {@link AISMessage09Impl#getLatitude}, 
     * {@link AISMessage18#getLatitude}, {@link AISMessage19#getLatitude}, or {@link AISMessage21#getLatitude}.
	 * @return true if the latitude is available
	 */
	public static boolean isAvailable(int value)
	{
		return !(value == UtilsLatitude27.DEFAULTVALUE); 
	}

	/** Converts the latitude value (in 1/10000 minutes) to degrees.
	 * @param value the latitude value that is returned by 
     * {@link AISMessagePositionReport#getLatitude}, {@link AISMessageUTCReport#getLatitude}, {@link AISMessage09Impl#getLatitude}, 
     * {@link AISMessage18#getLatitude}, {@link AISMessage19#getLatitude}, or {@link AISMessage21#getLatitude}.
	 * @return the latitude value in degrees
	 */
	public static double toDegrees ( int value )
	{
		return value * UtilsLatitude27.MultiplicationFactorToDegrees; 
	}
	
	/** Converts the latitude value (in 1/10000 minutes) to radians.
	 * @param value the latitude value that is returned by 
     * {@link AISMessagePositionReport#getLatitude}, {@link AISMessageUTCReport#getLatitude}, {@link AISMessage09Impl#getLatitude}, 
     * {@link AISMessage18#getLatitude}, {@link AISMessage19#getLatitude}, or {@link AISMessage21#getLatitude}.
	 * @return the latitude value in radians
	 */
	//TODO: should be removed?  use Math.toRadians(angdeg) instead (but with performance overhead?)
	public static double toRadians ( int value )
	{
		return value * UtilsLatitude27.MultiplicationFactorToRadians;
	}

	/** This constructor is made private to indicate that clients should not construct instances of this class. */
	private UtilsLatitude27 () {
	}
}