package nl.esi.metis.aisparser;

/** This class provides functions to analyze the (18-bit signed integer) longitude value that is returned by 
 * {@link AISMessage17#getLongitude}, {@link AISMessage27#getLongitude}, 
 * {@link DestinationArea#longitudeUpperRightCorner}, or {@link DestinationArea#longitudeLowerLeftCorner}.
 * @author Pierre van de Laar
 * @author Pierre America
 * @author Brian C. Lane
 */
public class UtilsLongitude18 {
	/** The number of integer units in a minute (as a measure of angle). 
	 * The fact that this value is 10 means that the integer longitude values we are dealing with express an angle in units of 1/10 minute.	 */
	private static final long UnitsPerMinute = 10; //10 units/minute

	/** The number of minutes in a degree (as a measure of angle). */
	private static final long MinutesPerDegree = 60; //60 minutes/degree
	
	/** The longitude value that is used to signal unavailability, according to the AIS standard. */
	private static final long DEFAULTVALUE = 181 * MinutesPerDegree * UnitsPerMinute ;	//181 degrees 

	/** The minimum valid longitude value. */
	private static final long MINVALUE = -180 * MinutesPerDegree * UnitsPerMinute; //-180 degrees

	/** The maximum valid longitude value. */
	private static final long MAXVALUE =  180 * MinutesPerDegree * UnitsPerMinute; //+180 degrees

	/** The multiplication factor to convert longitude values to degrees. */
	private static final double MultiplicationFactorToDegrees = 1.0 / (MinutesPerDegree * UnitsPerMinute);

	/** The multiplication factor to convert longitude values to radians. */
	private static final double MultiplicationFactorToRadians = Math.PI / (180 * MinutesPerDegree * UnitsPerMinute);
	
	/** Checks if the longitude value is semantically correct (according to the standard). 
	 * This means that it is either a valid longitude value or the standard value indicating unavailability.
	 * @param value the longitude value that is returned by 
     * {@link AISMessage17#getLongitude}, {@link AISMessage27#getLongitude}, 
     * {@link DestinationArea#longitudeUpperRightCorner}, or {@link DestinationArea#longitudeLowerLeftCorner}.
	 * @return true if the value is semantically correct
	 */
	//TODO: message 17 & 22 mention explicitly a default /not available value, but messages 23 and 27 do not. Should they NOT accept the default value as semantically correct? 
	public static boolean isSemanticallyCorrectValue( long value )
	{
		return ( ( UtilsLongitude18.MINVALUE <= value ) && ( value <= UtilsLongitude18.MAXVALUE ) ) || (value == UtilsLongitude18.DEFAULTVALUE);
	}
	
	/** Checks if the longitude value is available.
	 * This means that it is a value in between the minimum and maximum valid longitude values.
	 * @param value the longitude value that is returned by 
     * {@link AISMessage17#getLongitude}, {@link AISMessage27#getLongitude}, 
     * {@link DestinationArea#longitudeUpperRightCorner}, or {@link DestinationArea#longitudeLowerLeftCorner}.
	 * @return true if the longitude is available
	 */
	public static boolean isAvailable(long value)
	{
		return !(value == UtilsLongitude18.DEFAULTVALUE); 
	}

	/** Converts the longitude value (in 1/10 minutes) to degrees.
	 * @param value the longitude value that is returned by 
     * {@link AISMessage17#getLongitude}, {@link AISMessage27#getLongitude}, 
     * {@link DestinationArea#longitudeUpperRightCorner}, or {@link DestinationArea#longitudeLowerLeftCorner}.
	 * @return the longitude value in degrees
	 */
	public static double toDegrees ( long value )
	{
		return value * UtilsLongitude18.MultiplicationFactorToDegrees; 
	}
	
	/** Converts the longitude value (in 1/10 minutes) to radians.
	 * @param value the longitude value that is returned by 
     * {@link AISMessage17#getLongitude}, {@link AISMessage27#getLongitude}, 
     * {@link DestinationArea#longitudeUpperRightCorner}, or {@link DestinationArea#longitudeLowerLeftCorner}.
	 * @return the longitude value in radians
	 */
	//TODO: should be removed?  use Math.toRadians(angdeg) instead (but with performance overhead?)
	public static double toRadians ( long value )
	{
		return value * UtilsLongitude18.MultiplicationFactorToRadians;
	}

	/** This constructor is made private to indicate that clients should not construct instances of this class. */
	private UtilsLongitude18 () {
	}
}