package nl.esi.metis.aisparser;

/** This class provides functions to analyze the (28-bit signed integer) longitude value that is returned by 
 * {@link AISMessagePositionReport#getLongitude}, {@link AISMessageUTCReport#getLongitude}, {@link AISMessage09Impl#getLongitude}, 
 * {@link AISMessage18#getLongitude}, {@link AISMessage19#getLongitude}, or {@link AISMessage21#getLongitude}.
 * @author Pierre van de Laar
 * @author Pierre America
 * @author Brian C. Lane
 */
public class UtilsLongitude28 {
	/** The number of integer units in a minute (as a measure of angle). 
	 * The fact that this value is 10000 means that the integer longitude values we are dealing with express an angle in units of 1/10000 minute.	 */
	private static final int UnitsPerMinute = 10000; //10 000 units/minute

	/** The number of minutes in a degree (as a measure of angle). */
	private static final int MinutesPerDegree = 60; //60 minutes/degree
	
	/** The longitude value that is used to signal unavailability, according to the AIS standard. */
	private static final int DEFAULTVALUE = 181 * MinutesPerDegree * UnitsPerMinute ;	//181 degrees 

	/** The minimum valid longitude value. */
	private static final int MINVALUE = -180 * MinutesPerDegree * UnitsPerMinute; //-180 degrees

	/** The maximum valid longitude value. */
	private static final int MAXVALUE =  180 * MinutesPerDegree * UnitsPerMinute; //+180 degrees

	/** The multiplication factor to convert longitude values to degrees. */
	private static final double MultiplicationFactorToDegrees = 1.0 / (MinutesPerDegree * UnitsPerMinute);

	/** The multiplication factor to convert longitude values to radians. */
	private static final double MultiplicationFactorToRadians = Math.PI / (180 * MinutesPerDegree * UnitsPerMinute);

	/** Checks if the longitude value is semantically correct (according to the standard). 
	 * This means that it is either a valid longitude value or the standard value indicating unavailability.
	 * @param value the longitude value that is returned by 
     * {@link AISMessagePositionReport#getLongitude}, {@link AISMessageUTCReport#getLongitude}, {@link AISMessage09Impl#getLongitude}, 
     * {@link AISMessage18#getLongitude}, {@link AISMessage19#getLongitude}, or {@link AISMessage21#getLongitude}.
	 * @return true if the value is semantically correct
	 */
	public static boolean isSemanticallyCorrectValue( int value )
	{
		return ( ( UtilsLongitude28.MINVALUE <= value ) && ( value <= UtilsLongitude28.MAXVALUE ) ) || (value == UtilsLongitude28.DEFAULTVALUE);
	}
	
	/** Checks if the longitude value is available.
	 * This means that it is a value in between the minimum and maximum valid longitude values.
	 * @param value the longitude value that is returned by 
     * {@link AISMessagePositionReport#getLongitude}, {@link AISMessageUTCReport#getLongitude}, {@link AISMessage09Impl#getLongitude}, 
     * {@link AISMessage18#getLongitude}, {@link AISMessage19#getLongitude}, or {@link AISMessage21#getLongitude}.
	 * @return true if the longitude is available
	 */
	public static boolean isAvailable(int value)
	{
		return !(value == UtilsLongitude28.DEFAULTVALUE); 
	}

	/** Converts the longitude value (in 1/10000 minutes) to degrees.
	 * @param value the longitude value that is returned by 
     * {@link AISMessagePositionReport#getLongitude}, {@link AISMessageUTCReport#getLongitude}, {@link AISMessage09Impl#getLongitude}, 
     * {@link AISMessage18#getLongitude}, {@link AISMessage19#getLongitude}, or {@link AISMessage21#getLongitude}.
	 * @return the longitude value in degrees
	 */
	public static double toDegrees ( int value )
	{
		return value * UtilsLongitude28.MultiplicationFactorToDegrees; 
	}
	
	/** Converts the longitude value (in 1/10000 minutes) to radians.
	 * @param value the longitude value that is returned by 
     * {@link AISMessagePositionReport#getLongitude}, {@link AISMessageUTCReport#getLongitude}, {@link AISMessage09Impl#getLongitude}, 
     * {@link AISMessage18#getLongitude}, {@link AISMessage19#getLongitude}, or {@link AISMessage21#getLongitude}.
	 * @return the longitude value in radians
	 */
	//TODO: should be removed?  use Math.toRadians(angdeg) instead (but with performance overhead?)
	public static double toRadians ( int value )
	{
		return value * UtilsLongitude28.MultiplicationFactorToRadians;
	}

	/** This constructor is made private to indicate that clients should not construct instances of this class. */
	private UtilsLongitude28 () {
	}
}